=== modified file 'NEWS'
--- NEWS	2007-11-30 22:43:21 +0000
+++ NEWS	2007-12-01 21:59:46 +0000
@@ -14,6 +14,10 @@
 
   BUGFIXES:
 
+   * Catch OSError 17 (file exists) in final phase of tree transform and show
+     filename to user.
+     (Alexander Belchenko, #111758)
+
    * Catch ShortReadvErrors while using pycurl. Also make readv more robust by
      allowing multiple GET requests to be issued if too many ranges are
      required.

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-11-29 07:12:42 +0000
+++ bzrlib/tests/__init__.py	2007-12-01 21:59:46 +0000
@@ -2771,3 +2771,29 @@
         return 'FTPServer'
 
 FTPServerFeature = _FTPServerFeature()
+
+
+class _CaseInsensitiveFilesystemFeature(Feature):
+    """Check if underlined filesystem is case-insensitive
+    (e.g. on Windows, Cygwin, MacOS)
+    """
+
+    def _probe(self):
+        if TestCaseWithMemoryTransport.TEST_ROOT is None:
+            root = osutils.mkdtemp(prefix='testbzr-', suffix='.tmp')
+            TestCaseWithMemoryTransport.TEST_ROOT = root
+        else:
+            root = TestCaseWithMemoryTransport.TEST_ROOT
+        tdir = osutils.mkdtemp(prefix='case-sensitive-probe-', suffix='',
+            dir=root)
+        name_a = osutils.pathjoin(tdir, 'a')
+        name_A = osutils.pathjoin(tdir, 'A')
+        os.mkdir(name_a)
+        result = osutils.isdir(name_A)
+        _rmtree_temp_dir(tdir)
+        return result
+
+    def feature_name(self):
+        return 'case-insensitive filesystem'
+
+CaseInsensitiveFilesystemFeature = _CaseInsensitiveFilesystemFeature()

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2007-11-28 23:08:40 +0000
+++ bzrlib/tests/test_transform.py	2007-12-01 21:59:46 +0000
@@ -36,6 +36,7 @@
 from bzrlib.osutils import file_kind, pathjoin
 from bzrlib.merge import Merge3Merger
 from bzrlib.tests import (
+    CaseInsensitiveFilesystemFeature,
     SymlinkFeature,
     TestCase,
     TestCaseInTempDir,
@@ -1088,8 +1089,24 @@
         self.callDeprecated([txt], change_entry, None, None, None, None, None,
             None, None, None)
 
+    def test_case_insensitive_clash(self):
+        self.requireFeature(CaseInsensitiveFilesystemFeature)
+        def tt_helper():
+            wt = self.make_branch_and_tree('.')
+            tt = TreeTransform(wt)  # TreeTransform obtains write lock
+            try:
+                tt.new_file('foo', tt.root, 'bar')
+                tt.new_file('Foo', tt.root, 'spam')
+                tt.apply(no_conflicts=True)
+            finally:
+                wt.unlock()
+        err = self.assertRaises(errors.FileExists, tt_helper)
+        self.assertContainsRe(str(err),
+            "^File exists: .+/foo")
+
 
 class TransformGroup(object):
+
     def __init__(self, dirname, root_id):
         self.name = dirname
         os.mkdir(dirname)
@@ -1462,6 +1479,7 @@
         object.__init__(self)
         self.name = "name"
 
+
 class TestGetBackupName(TestCase):
     def test_get_backup_name(self):
         tt = MockTransform()

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2007-11-27 13:24:44 +0000
+++ bzrlib/transform.py	2007-12-01 21:59:46 +0000
@@ -1878,7 +1878,12 @@
 
     def rename(self, from_, to):
         """Rename a file from one path to another.  Functions like os.rename"""
-        os.rename(from_, to)
+        try:
+            os.rename(from_, to)
+        except OSError, e:
+            if e.errno == errno.EEXIST:
+                raise errors.FileExists(to, str(e))
+            raise
         self.past_renames.append((from_, to))
 
     def pre_delete(self, from_, to):

